@@ -4,8 +4,10 @@
 // This file is placed under the conditions of the GNU Library
 // General Public License, version 2, or any later version.
 // See file COPYING for information on distribution conditions.
-
-/* File for parsing top-level /proc entities. */
+//
+// File for parsing top-level /proc entities. */
+//
+// June 2003, Fabian Frederick, disk and slab info
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -172,7 +174,9 @@ static void old_Hertz_hack(void){
 #define AT_CLKTCK       17    /* frequency of times() */
 #endif
 
-extern char** environ;
+#define NOTE_NOT_FOUND 42
+
+//extern char** environ;
 
 /* for ELF executables, notes are pushed before environment and args */
 static unsigned long find_elf_note(unsigned long findme){
@@ -182,7 +186,7 @@ static unsigned long find_elf_note(unsigned long findme){
     if(ep[0]==findme) return ep[1];
     ep+=2;
   }
-  return 42;
+  return NOTE_NOT_FOUND;
 }
 
 static void init_libproc(void) __attribute__((constructor));
@@ -195,12 +199,13 @@ static void init_libproc(void){
 
   if(linux_version_code > LINUX_VERSION(2, 4, 0)){ 
     Hertz = find_elf_note(AT_CLKTCK);
-    if(Hertz!=42) return;
+    if(Hertz!=NOTE_NOT_FOUND) return;
     fprintf(stderr, "2.4 kernel w/o ELF notes? -- report to albert@users.sf.net\n");
   }
   old_Hertz_hack();
 }
 
+#if 0
 /***********************************************************************
  * The /proc filesystem calculates idle=jiffies-(user+nice+sys) and we
  * recover jiffies by adding up the 4 or 5 numbers we are given. SMP kernels
@@ -211,19 +216,23 @@ static void init_libproc(void){
 #define NAN (-0.0)
 #endif
 #define JT unsigned long long
-void five_cpu_numbers(double *restrict uret, double *restrict nret, double *restrict sret, double *restrict iret, double *restrict wret){
-    double tmp_u, tmp_n, tmp_s, tmp_i, tmp_w;
+void seven_cpu_numbers(double *restrict uret, double *restrict nret, double *restrict sret, double *restrict iret, double *restrict wret, double *restrict xret, double *restrict yret){
+    double tmp_u, tmp_n, tmp_s, tmp_i, tmp_w, tmp_x, tmp_y;
     double scale;  /* scale values to % */
-    static JT old_u, old_n, old_s, old_i, old_w;
-    JT new_u, new_n, new_s, new_i, new_w;
+    static JT old_u, old_n, old_s, old_i, old_w, old_x, old_y;
+    JT new_u, new_n, new_s, new_i, new_w, new_x, new_y;
     JT ticks_past; /* avoid div-by-0 by not calling too often :-( */
 
     tmp_w = 0.0;
     new_w = 0;
+    tmp_x = 0.0;
+    new_x = 0;
+    tmp_y = 0.0;
+    new_y = 0;
  
     FILE_TO_BUF(STAT_FILE,stat_fd);
-    sscanf(buf, "cpu %Lu %Lu %Lu %Lu %Lu", &new_u, &new_n, &new_s, &new_i, &new_w);
-    ticks_past = (new_u+new_n+new_s+new_i+new_w)-(old_u+old_n+old_s+old_i+old_w);
+    sscanf(buf, "cpu %Lu %Lu %Lu %Lu %Lu %Lu %Lu", &new_u, &new_n, &new_s, &new_i, &new_w, &new_x, &new_y);
+    ticks_past = (new_u+new_n+new_s+new_i+new_w+new_x+new_y)-(old_u+old_n+old_s+old_i+old_w+old_x+old_y);
     if(ticks_past){
       scale = 100.0 / (double)ticks_past;
       tmp_u = ( (double)new_u - (double)old_u ) * scale;
@@ -231,25 +240,34 @@ void five_cpu_numbers(double *restrict uret, double *restrict nret, double *rest
       tmp_s = ( (double)new_s - (double)old_s ) * scale;
       tmp_i = ( (double)new_i - (double)old_i ) * scale;
       tmp_w = ( (double)new_w - (double)old_w ) * scale;
+      tmp_x = ( (double)new_x - (double)old_x ) * scale;
+      tmp_y = ( (double)new_y - (double)old_y ) * scale;
     }else{
       tmp_u = NAN;
       tmp_n = NAN;
       tmp_s = NAN;
       tmp_i = NAN;
       tmp_w = NAN;
+      tmp_x = NAN;
+      tmp_y = NAN;
     }
     SET_IF_DESIRED(uret, tmp_u);
     SET_IF_DESIRED(nret, tmp_n);
     SET_IF_DESIRED(sret, tmp_s);
     SET_IF_DESIRED(iret, tmp_i);
     SET_IF_DESIRED(wret, tmp_w);
+    SET_IF_DESIRED(iret, tmp_x);
+    SET_IF_DESIRED(wret, tmp_y);
     old_u=new_u;
     old_n=new_n;
     old_s=new_s;
     old_i=new_i;
     old_w=new_w;
+    old_i=new_x;
+    old_w=new_y;
 }
 #undef JT
+#endif
 
 /***********************************************************************/
 void loadavg(double *restrict av1, double *restrict av5, double *restrict av15) {
@@ -323,7 +341,7 @@ static void getrunners(unsigned int *restrict running, unsigned int *restrict bl
 
 /***********************************************************************/
 
-void getstat(jiff *restrict cuse, jiff *restrict cice, jiff *restrict csys, jiff *restrict cide, jiff *restrict ciow,
+void getstat(jiff *restrict cuse, jiff *restrict cice, jiff *restrict csys, jiff *restrict cide, jiff *restrict ciow, jiff *restrict cxxx, jiff *restrict cyyy,
 	     unsigned long *restrict pin, unsigned long *restrict pout, unsigned long *restrict s_in, unsigned long *restrict sout,
 	     unsigned *restrict intr, unsigned *restrict ctxt,
 	     unsigned int *restrict running, unsigned int *restrict blocked,
@@ -343,9 +361,11 @@ void getstat(jiff *restrict cuse, jiff *restrict cice, jiff *restrict csys, jiff
   read(fd,buff,BUFFSIZE-1);
   *intr = 0; 
   *ciow = 0;  /* not separated out until the 2.5.41 kernel */
+  *cxxx = 0;  /* not separated out until the 2.6.0-test4 kernel */
+  *cyyy = 0;  /* not separated out until the 2.6.0-test4 kernel */
 
   b = strstr(buff, "cpu ");
-  if(b) sscanf(b,  "cpu  %Lu %Lu %Lu %Lu %Lu", cuse, cice, csys, cide, ciow);
+  if(b) sscanf(b,  "cpu  %Lu %Lu %Lu %Lu %Lu %Lu %Lu", cuse, cice, csys, cide, ciow, cxxx, cyyy);
 
   b = strstr(buff, "page ");
   if(b) sscanf(b,  "page %lu %lu", pin, pout);
@@ -651,3 +671,152 @@ nextline:
   }
 }
 
+///////////////////////////////////////////////////////////////////////
+// based on Fabian Frederick's /proc/diskstats parser
+
+static unsigned int getFileLines(const char* szFile){
+  char szBuffer[1024];
+  FILE *fdiskStat;
+  int lines=0;
+  if ((fdiskStat=fopen (szFile,"rb"))){
+    while (fgets(szBuffer, 1024, fdiskStat)){
+      lines++;
+    }
+    fclose(fdiskStat);
+  } 
+  return lines;
+}
+
+/////////////////////////////////////////////////////////////////////////////
+
+unsigned int getpartitions_num(struct disk_stat *disks, int ndisks){
+  int i=0;
+  int partitions=0;
+
+  for (i=0;i<ndisks;i++){
+	partitions+=disks[i].partitions;
+  }
+  return partitions;
+
+}
+
+/////////////////////////////////////////////////////////////////////////////
+
+unsigned int getdiskstat(struct disk_stat **disks, struct partition_stat **partitions){
+  FILE* fd;
+  int units,
+      i,
+      disk_type,
+      disk_num,
+      cDisk=0,
+      cPartition=0;
+
+  *disks = NULL;
+  *partitions = NULL;
+  buff[BUFFSIZE-1] = 0; 
+  units = getFileLines("/proc/diskstats");
+  fd = fopen("/proc/diskstats", "rb");
+  if(!fd) crash("/proc/diskstats");
+
+  for (i=0; i<units; i++){
+    if (!fgets(buff,BUFFSIZE-1,fd)){
+      fclose(fd);
+      crash("/proc/diskstats");
+    }
+    sscanf(buff, "    %d    %d", &disk_type, &disk_num);
+    if (disk_num == 0){
+      (*disks) = realloc(*disks, (cDisk+1)*sizeof(struct disk_stat));
+      sscanf(buff,  "   %d    %*d %15s %u %u %llu %u %u %u %llu %u %u %u %u",
+        &(*disks)[cDisk].disk_type,
+        //&unused,
+        (*disks)[cDisk].disk_name,
+        &(*disks)[cDisk].reads,
+        &(*disks)[cDisk].merged_reads,
+        &(*disks)[cDisk].reads_sectors,
+        &(*disks)[cDisk].milli_reading,
+        &(*disks)[cDisk].writes,
+        &(*disks)[cDisk].merged_writes,
+        &(*disks)[cDisk].written_sectors,
+        &(*disks)[cDisk].milli_writing,
+        &(*disks)[cDisk].inprogress_IO,
+        &(*disks)[cDisk].milli_spent_IO,
+        &(*disks)[cDisk].weighted_milli_spent_IO
+      );
+        (*disks)[cDisk].partitions=0;
+      cDisk++;
+    }else{
+      (*partitions) = realloc(*partitions, (cPartition+1)*sizeof(struct partition_stat));
+      fflush(stdout);
+      sscanf(buff,  "   %d    %d %15s %u %llu %u %u",
+        &(*partitions)[cPartition].disk_type,
+        &(*partitions)[cPartition].partition_num,
+        (*partitions)[cPartition].partition_name,
+        &(*partitions)[cPartition].reads,
+        &(*partitions)[cPartition].reads_sectors,
+        &(*partitions)[cPartition].writes,
+        &(*partitions)[cPartition].requested_writes
+      );
+      (*partitions)[cPartition++].parent_disk = &((*disks)[cDisk-1]);
+      (*disks)[cDisk-1].partitions++;	
+    }
+  }
+  fclose(fd);
+  return cDisk;
+}
+
+/////////////////////////////////////////////////////////////////////////////
+// based on Fabian Frederick's /proc/slabinfo parser
+
+unsigned int getslabinfo (struct slab_cache **slab){
+  FILE* fd;
+  int cSlab = 0;
+  buff[BUFFSIZE-1] = 0; 
+  *slab = NULL;
+  fd = fopen("/proc/slabinfo", "rb");
+  if(!fd) crash("/proc/slabinfo");
+  while (fgets(buff,BUFFSIZE-1,fd)){
+    if(!memcmp("slabinfo - version:",buff,19)) continue; // skip header
+    if(*buff == '#')                           continue; // skip comments
+    (*slab) = realloc(*slab, (cSlab+1)*sizeof(struct slab_cache));
+    sscanf(buff,  "%47s %u %u %u %u",  // allow 47; max seen is 24
+      (*slab)[cSlab].name,
+      &(*slab)[cSlab].active_objs,
+      &(*slab)[cSlab].num_objs,
+      &(*slab)[cSlab].objsize,
+      &(*slab)[cSlab].objperslab
+    ) ;
+    cSlab++;
+  }
+  fclose(fd);
+  return cSlab;
+}
+
+///////////////////////////////////////////////////////////////////////////
+
+unsigned get_pid_digits(void){
+  char pidbuf[24];
+  char *endp;
+  long rc;
+  int fd;
+  static unsigned ret;
+
+  if(ret) goto out;
+  ret = 5;
+  fd = open("/proc/sys/kernel/pid_max", O_RDONLY);
+  if(fd==-1) goto out;
+  rc = read(fd, pidbuf, sizeof pidbuf);
+  close(fd);
+  if(rc<3) goto out;
+  pidbuf[rc] = '\0';
+  rc = strtol(pidbuf,&endp,10);
+  if(rc<42) goto out;
+  if(*endp && *endp!='\n') goto out;
+  rc--;  // the pid_max value is really the max PID plus 1
+  ret = 0;
+  while(rc){
+    rc /= 10;
+    ret++;
+  }
+out:
+  return ret;
+}
