@@ -270,7 +270,6 @@ static const char *parse_sysv_option(void){
     case 'M':  /* someday, maybe, we will have MAC like SGI's Irix */
       trace("-M Print security label for Mandatory Access Control.\n");
       format_modifiers |= FM_M;
-      return "Sorry, no Mandatory Access Control support.";
       break;
     case 'N':
       trace("-N negates.\n");
@@ -568,6 +567,10 @@ static const char *parse_bsd_option(void){
       trace("X Old Linux i386 register format\n");
       format_flags |= FF_LX;
       break;
+    case 'Z':  /* FreeBSD does MAC like SGI's Irix does it */
+      trace("Z Print security label for Mandatory Access Control.\n");
+      format_modifiers |= FM_M;
+      break;
     case 'a':
       trace("a Select all w/tty, including other users\n");
       simple_select |= SS_B_a;
@@ -737,7 +740,6 @@ static const char *parse_gnu_option(void){
   gnu_table_struct *found;
   static const gnu_table_struct gnu_table[] = {
   {"Group",         &&case_Group},       /* rgid */
-  {"SID",           &&case_secsid},
   {"User",          &&case_User},        /* ruid */
   {"cols",          &&case_cols},
   {"columns",       &&case_columns},
@@ -765,7 +767,6 @@ static const char *parse_gnu_option(void){
   {"pid",           &&case_pid},
   {"ppid",          &&case_ppid},
   {"rows",          &&case_rows},
-  {"secsid",        &&case_secsid},
   {"sid",           &&case_sid},
   {"sort",          &&case_sort},
   {"tty",           &&case_tty},
@@ -954,10 +955,6 @@ static const char *parse_gnu_option(void){
     trace("--context\n");
     format_flags |= FF_Fc;
     return NULL;
-  case_secsid:
-     trace("--secsid\n");
-     format_flags |= FF_Fs;
-     return NULL;
 }
 
 /*************** process trailing PIDs  **********************/
